;;; cogroo.el --- Utiliza o corretor gramatical CoGrOO para corrigir textos
;; Copyright (C) 2010 Thiago Maciel Batista
          
;; Author: Thiago Maciel Batista <thiguns@gmail.com>
;; Created: 04 Aug 2010
;; Version: 1.0
;; Keywords: corretor cogroo

;; This file is NOT part of GNU Emacs.

;; This program is free software: you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation, either version 3 of
;; the License, or (at your option) any later version.
          
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
          
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; A real fast way to track down library files in your load-path.
;; Basically I've found NSF's fff.el a little buggy and TM's
;; find-library.el way to slow.  This is a quick hack that uses Gareth
;; Rees's `read-library' package. This all of course assumes you are
;; only interested in files in your load-path.

;;; Code:

;;***inicia o corretor em java***

(defvar cogroo-corrige-automaticamente nil)
(defvar cogroo-inicia-automaticamente nil)
(defvar cogroo-iniciado nil)
(defvar cogroo-corretor nil)
(defvar cogroo-map (make-sparse-keymap))



;;funcall function &rest arguments
;;(funcall 'cogroo-inicia)

(defun cogroo-inicia ()
  (interactive)
  (message "Está iniciando")
  (cogroo-inicia-corretor))
  ;;(define-key cogroo-map [mouse-1] 'cogroo-substitui-erro)
  ;;(define-key cogroo-map [mouse-3] 'cogroo-substitui-erro-2)) 

(defun cogroo-inicia-automaticamente ()
  (if (equal cogroo-inicia-automaticamente t)
      (cogroo-inicia-corretor)))
  
(defun cogroo-inicia-corretor ()
  (if (equal cogroo-iniciado nil)
    (progn
      (setq cogroo-corretor (start-process "corretor" nil "cogrooemacs"))
      (while (equal (accept-process-output cogroo-corretor) nil))
      (set-process-query-on-exit-flag (get-process "corretor") nil)
      (setq cogroo-iniciado t)
      (define-key cogroo-map [mouse-1] 'cogroo-substitui-erro)
      (define-key cogroo-map [mouse-3] 'cogroo-substitui-erro-2)
      ;;(define-key cogroo-map [down] 'cogroo)
      (if (equal cogroo-corrige-automaticamente t)
        (global-set-key [down] 'cogroo-corrige-automatico))
      (message "Corretor iniciado"))
   (message "O corretor já foi iniciado")))   
   
(funcall 'cogroo-inicia-automaticamente)    

(defun cogroo-corrige-automatico ()
  (interactive)
  
  (let (posicao distancia inicio fim linha (eh-paragrafo nil))
    (with-current-buffer (current-buffer)
      (setq inicio (line-beginning-position)) ;;inicio da linha atual
      (setq fim (line-end-position)) ;;fim da linha atual     
      (setq posicao (point)) ;;posicao do point na linha atual
      (setq distancia (- posicao inicio)) ;;distancia do point ate o inicio da linha atual      
      (if (not (equal inicio fim))
        ;;Está dentro de um parágrafo    
          (setq eh-paragrafo t))
      
      (forward-line 1) ;;pula para a proxima linha
      (setq inicio (line-beginning-position)) ;;inicio da nova linha
      (setq fim (line-end-position)) ;;fim da nova linha     
      (setq linha (buffer-substring-no-properties inicio fim)) ;;pega a linha     
      (if (looking-at paragraph-separate)
        (progn          
          (if (and (equal eh-paragrafo t) (equal cogroo-iniciado t))
            (progn             
              (save-excursion
                (goto-char posicao)
                (cogroo-corrige)))))
       (progn       
         (if (> (+ inicio distancia) fim)             
           (goto-char fim)
          (goto-char (+ inicio distancia))))))))



(defun cogroo-finaliza ()
  "mata o processo do corretor"
  (interactive)
  (if (equal cogroo-iniciado t)
  (progn
    (delete-process cogroo-corretor)
    (setq cogroo-iniciado nil)
    (setq cogroo-corretor nil)) 
   (message "O corretor nao foi iniciado")))

;;***pega um parágrafo ou uma região e manda para o corretor corrigir***

(defvar cogroo-comando "corrige
")

(defun cogroo-corrige ()
  "corrige um trecho do texto, que pode ser o parágrafo 
  em que o point está ou um texto selecionado"
  
  (interactive)
  (if (equal cogroo-iniciado nil)
    (message "Falta iniciar o corretor Cogroo")
   (progn
     (let (inicio final)
      (save-excursion
        (if (not mark-active) 
          (mark-paragraph))   
        (setq inicio (region-beginning))
        (setq final (region-end)))
      (cogroo-desmarca-erros inicio final)
      (cogroo-escreve-no-arquivo inicio final)
      (cogroo-manda-corrigir inicio)))))

(defun cogroo-escreve-no-arquivo (inicio fim)
  "Escreve o trecho de texto entre inicio e fim no arquivo /tmp/cogrootexto"
  (write-region inicio fim "/tmp/cogrootexto" nil 1))
       
(defun cogroo-manda-corrigir (inicio)    
  "Manda mensagem para o processo do corretor mandando corrigir o texto"
  (process-send-string cogroo-corretor cogroo-comando)
  (while (equal (accept-process-output cogroo-corretor) nil))
  (cogroo-marca-erros inicio))

;;;(global-set-key (kbd "C-x C-C") 'cogroo-corrige)

;;;*** função para substituir erros***

(defun cogroo-substitui-erro ()
  "Substitui um trecho marcado como incorreto 
  no buffer pela sugestão dada pelo corretor"
  (interactive)
  (let (posicao overlays overlay inicio final sugestoes)
    (setq posicao (posn-point (event-start last-input-event)))
    (setq overlays (overlays-at posicao))
    (setq overlay (car overlays))
    (setq inicio (overlay-start overlay))
    (setq final (overlay-end overlay))
    (setq sugestoes (overlay-get overlay 'sugestoes))
    (if (not (equal (car sugestoes) nil))
      (progn
        (delete-region inicio final)
        (save-excursion
          (goto-char inicio)
          (insert (car sugestoes)))))))

(defun cogroo-substitui-erro-2 ()
  "Substitui um trecho marcado como incorreto 
  no buffer pela segunda sugestão dada pelo corretor"
  (interactive)
  (let (posicao overlays overlay inicio final sugestoes)
    (setq posicao (posn-point (event-start last-input-event)))
    (setq overlays (overlays-at posicao))
    (setq overlay (car overlays))
    (setq inicio (overlay-start overlay))
    (setq final (overlay-end overlay))
    (setq sugestoes (overlay-get overlay 'sugestoes))
    (if (not (equal (cdr sugestoes) nil))
      (progn
        (delete-region inicio final)
        (save-excursion
          (goto-char inicio)
          (setq sugestoes (cdr sugestoes))
          (insert (car sugestoes)))))))




;;;***funções para marcar erros***

(defface cogroo-destaque-erro
  '((t (:bold t :underline "magenta"))) "destaque erro" :group 'cogroo-faces) 

(defface cogroo-mouse-face
  '((t (:bold t :underline "magenta" :background "blue3"))) "mouse face com sugestao" :group 'cogroo-faces)                
  
(defface cogroo-mouse-face-2
  '((t (:bold t :underline "magenta" :background "gray"))) "mouse face sem sugestao" :group 'cogroo-faces)  
    
(defun cogroo-marca-erros (inicio-paragrafo)
  "marca, no buffer, trechos incorretos contidos
  no parágrafo que começa em inicio-paragrafo. É usado overlays para destacar"
  (let ((erros nil) inicio fim informacoes sugestoes) 
    (setq erros (cogroo-le-erros))
    (while erros
      (setq inicio (car erros))
      (setq erros (cdr erros))
      (setq fim (car erros))
      (setq erros (cdr erros))
      (setq sugestoes nil)
      (setq informacoes "CoGrOO-Emacs\n")
      (while (not (equal (car erros) "*"))
        (setq informacoes (concat informacoes (car erros) "\n"))
        (setq erros (cdr erros)))
      (setq erros (cdr erros))  
      (while (not (equal (car erros) "**"))        
        (setq sugestoes (cons (car erros) sugestoes))
        (setq erros (cdr erros))) 
      (setq erros (cdr erros))     
      (setq informacoes (cogroo-completa-informacoes informacoes sugestoes))
      (cogroo-destaca-texto (+ (string-to-number inicio) inicio-paragrafo) (+ (string-to-number fim) inicio-paragrafo) informacoes sugestoes)))
  (cogroo-apaga-arquivo))
  
(defun cogroo-completa-informacoes (informacoes sugestoes)
  "Concatena a string informacoes com as sugestoes.
  Caso não exista sugestão, concatena informacoes com um aviso"
  (if (not (null sugestoes))
    (progn
      (let ((tem_duas_sugestoes nil))
        (if (not (equal (cdr sugestoes) nil))
          (setq tem_duas_sugestoes t))
        (setq informacoes (concat informacoes "Sugestões: '"))
        (while sugestoes
          (setq informacoes (concat informacoes "[" (car sugestoes) "]  "))
          (setq sugestoes (cdr sugestoes)))
        (setq informacoes (concat informacoes "\n"))
        (if (equal tem_duas_sugestoes t)
          (progn
            (setq informacoes (concat informacoes "*** Clique com o botão esquerdo do mouse para aceitar a primeira sugestão***\n"))
            (setq informacoes (concat informacoes "*** Clique com o botão direito do mouse para aceitar a segunda sugestão***\n")))
       (setq informacoes (concat informacoes "*** Clique com o botão esquerdo do mouse para aceitar a sugestão***\n")))))
   (setq informacoes (concat informacoes "***Não há sugestões para esse caso***\n")))
  informacoes)
       
(defun cogroo-le-erros () 
  "Devolve uma lista com as linhas do arquivo cogrooerros" 
  (with-temp-buffer 
    (while (equal (condition-case nil (insert-file-contents "/tmp/cogrooerros") (error nil)) nil))
    (split-string (buffer-string) "\n" t)))
    
(defun cogroo-apaga-arquivo ()
   (delete-file "/tmp/cogrooerros"))  

(defun cogroo-destaca-texto (pos1 pos2 informacoes sugestoes)
  "Destaca o texto entre as posiçoes pos1 e pos2 do buffer usando overlay"
  (let (overlay)
    (setq overlay (cogroo-adiciona-overlay pos1 pos2 'cogroo-destaque-erro informacoes))
    ;;(if (not (equal sugestao ""))
    (if (not (equal (car sugestoes) nil))    
      (overlay-put overlay 'mouse-face 'cogroo-mouse-face)
     (overlay-put overlay 'mouse-face 'cogroo-mouse-face-2))
    (overlay-put overlay 'keymap cogroo-map)  
    (overlay-put overlay 'sugestoes sugestoes)))
    
(defun cogroo-adiciona-overlay (inicio final face informacoes)
  (let ((overlay (make-overlay inicio final)))
    (overlay-put overlay 'face face)
    (overlay-put overlay 'help-echo informacoes)
    overlay))

;;;***Função para desmarcar erros***
 
(defun cogroo-desmarca-erros (inicio final)
  "Remove todos overlays o buffer corrente entre inicio e final"
  (remove-overlays inicio final 'face 'cogroo-destaque-erro))         
 

;;byte-compile-file filename &optional load          para compilar filename
;;(provide 'cogroo)
;;; cogroo.el ends here  
